/*
 * playlistaspectsearchfilter.cpp
 *
 *  Created on: 18.04.2010
 *      Author: darkstar
 */

#include "playlistaspectsearchfilter.h"
#include "sqlite3.h"
#include "debug.h"
#include "compathack.h"

PlayListAspectSearchFilter::PlayListAspectSearchFilter()
	:	PlayListAspectExtension(),
		filterText_(""),
		dbFilterText_(QString::null)
{

}

PlayListAspectSearchFilter::~PlayListAspectSearchFilter()
{

}

void PlayListAspectSearchFilter::setFilterText(const QString &filter)
{
	if (filter != filterText_)
	{
		filterText_ = filter;
		execute();
	}
}

QString PlayListAspectSearchFilter::outputViewSource()
{
	return playListAspect()->prefix() + "textfilter";
}

void PlayListAspectSearchFilter::reset()
{
	dbFilterText_ = QString::null;
	filterText_ = "";
}

void PlayListAspectSearchFilter::execute()
{
	QString inputView = inputViewSource();
	QString outputView = outputViewSource();

	if (dbFilterText_.isNull() || dbFilterText_ != filterText_)
	{
		QString query =
			"DROP TABLE IF EXISTS " + outputView + "; "
			"CREATE TEMPORARY TABLE " + outputView + " AS "
			"SELECT " + inputView + ".playlist_id as playlist_id, " + inputView + ".media_id as media_id, idx, track, title, album, artist, genre, playtime, filesize "
			"FROM " + inputView + ", media "
			"WHERE " + inputView + ".media_id = media.media_id ";

		if (filterText_ != "")
		{
			QString conditionString = "";
			QString safeSearchString = "";

			// split input filter string into parts so we can build separate conditions for each...
			QStringList searchStrings = QStringList::split(' ', filterText_, false);
			int searchStringCount = searchStrings.count();

			// build condition string for view...
			for (int i = 0; i < searchStringCount; ++i)
			{
				safeSearchString = dbSafeString("%" + searchStrings[i] + "%");
				//conditionString += "(title LIKE '" + safeSearchString + "' OR artist LIKE '" + safeSearchString + "' OR album LIKE '" + safeSearchString + "')";

				// the following is faster but less accurate:
				conditionString += "(title || album || artist LIKE '" + safeSearchString + "')";

				// only append AND if this is not the last search string
				if (i < searchStringCount -1)
					conditionString += " AND ";
			}

			query += "AND (" + conditionString + ") ";
		}

		query += ";";

		DPRINTF("%s", (const char *)query.toUtf8());

		DTIMERINIT(temp);
		sqlite3_exec(playListAspect()->playList()->mediaDatabase()->db(), query.toUtf8(), NULL, NULL, NULL);

		// the schema changed, so we need to finalize and re-prepare statements...
		playListAspect()->playList()->mediaDatabase()->ensurePreparedStatementsAreValid();

		DTIMERPRINT(temp, "query");

		dbFilterText_ = filterText_;
	}

	PlayListAspectExtension::execute();
}

void PlayListAspectSearchFilter::saveExtensionState(QStringList &dst)
{
	dst.clear();
	dst.append(filterText_);
}

void PlayListAspectSearchFilter::loadExtensionState(QStringList &src)
{
	if (src.count() == 1)
		setFilterText(src[0]);
}
