/*
 * playlistaspect.h
 *
 *  Created on: 15.04.2010
 *      Author: darkstar
 */

#ifndef PLAYLISTASPECT_H_
#define PLAYLISTASPECT_H_

#include <qobject.h>

#include "playlist.h"

class PlayListAspectExtension;

#ifdef QT4
typedef QList<Media *> MediaList;
typedef QList<PlayListAspectExtension *> PlayListAspectExtensionList;
#else
typedef QList<Media> MediaList;
typedef QList<PlayListAspectExtension> PlayListAspectExtensionList;
#endif

class PlayListAspect;

class PlayListAspectExtension
{
public:
	PlayListAspectExtension() : playListAspect_(NULL) {};
	virtual ~PlayListAspectExtension() { }

	virtual void execute();
	virtual void reset();
	virtual QString outputViewSource() = 0;

	virtual bool isActive();
	virtual bool isFiltering();

	virtual const QString &identifier() { return outputViewSource(); }
	virtual void saveExtensionState(QStringList &dst);
	virtual void loadExtensionState(QStringList &src);

protected:
	friend class PlayListAspect;

	virtual void setPlayListAspect(PlayListAspect *playListAspect) { playListAspect_ = playListAspect; }
	PlayListAspect *playListAspect() { return playListAspect_; }

	virtual void setExtensionList(PlayListAspectExtensionList *extensionList) { extensionList_ = extensionList; }
	PlayListAspectExtensionList *extenstionList() { return extensionList_; }

	QString inputViewSource();

	PlayListAspectExtension *previousAspectExtension();
	PlayListAspectExtension *nextAspectExtension();

	virtual void extensionGraphExecutionFinished();

private:
	PlayListAspect *playListAspect_;
	PlayListAspectExtensionList *extensionList_;
};

class PlayListAspect : public QObject
{
	Q_OBJECT
public:
	PlayListAspect(PlayList *playList, const QString aspectName);
	virtual ~PlayListAspect();

	PlayList *playList() { return playList_; }
	const QString &name() { return aspectName_; }
	const QString &prefix() { return prefix_; }

	void registerExtension(PlayListAspectExtension *extension);
	void unregisterExtension(PlayListAspectExtension *extension);

	void loadFromDatabase(const QString &tableName);
	void saveToDatabase();

public slots:
	void update();

private:
	PlayList *playList_;
	QString aspectName_;
	QString prefix_;

	PlayListAspectExtensionList extensions_;

};

#endif /* PLAYLISTASPECT_H_ */
